@@ -1,23 +1,23 @@
 """Basic model line shapes and distribution functions."""
-from __future__ import division
 
-from numpy import arctan, cos, exp, log, pi, sin, sqrt, where
-from numpy.testing import assert_allclose
-from scipy.special import erf, erfc, gammaln, wofz
+from numpy import (arctan, cos, exp, finfo, float64, isnan, log, pi, real, sin,
+                   sqrt, where)
+from scipy.special import erf, erfc
 from scipy.special import gamma as gamfcn
+from scipy.special import wofz
 
 log2 = log(2)
 s2pi = sqrt(2*pi)
 spi = sqrt(pi)
 s2 = sqrt(2.0)
-tiny = 1.e-13
+tiny = finfo(float64).eps
 
 functions = ('gaussian', 'lorentzian', 'voigt', 'pvoigt', 'moffat', 'pearson7',
              'breit_wigner', 'damped_oscillator', 'dho', 'logistic', 'lognormal',
-             'students_t', 'expgaussian', 'donaich', 'skewed_gaussian',
-             'skewed_voigt', 'step', 'rectangle', 'erf', 'erfc', 'wofz',
-             'gamma', 'gammaln', 'exponential', 'powerlaw', 'linear',
-             'parabolic', 'sine', 'expsine')
+             'students_t', 'expgaussian', 'doniach', 'donaich', 'skewed_gaussian',
+             'skewed_voigt', 'thermal_distribution', 'step', 'rectangle',
+             'exponential', 'powerlaw', 'linear', 'parabolic', 'sine',
+             'expsine', 'split_lorentzian')
 
 
 def gaussian(x, amplitude=1.0, center=0.0, sigma=1.0):
@@ -27,7 +27,8 @@ def gaussian(x, amplitude=1.0, center=0.0, sigma=1.0):
         (amplitude/(s2pi*sigma)) * exp(-(1.0*x-center)**2 / (2*sigma**2))
 
     """
-    return (amplitude/(s2pi*sigma)) * exp(-(1.0*x-center)**2 / (2*sigma**2))
+    return ((amplitude/(max(tiny, s2pi*sigma)))
+            * exp(-(1.0*x-center)**2 / max(tiny, (2*sigma**2))))
 
 
 def lorentzian(x, amplitude=1.0, center=0.0, sigma=1.0):
@@ -37,7 +38,31 @@ def lorentzian(x, amplitude=1.0, center=0.0, sigma=1.0):
         (amplitude/(1 + ((1.0*x-center)/sigma)**2)) / (pi*sigma)
 
     """
-    return (amplitude/(1 + ((1.0*x-center)/sigma)**2)) / (pi*sigma)
+    return ((amplitude/(1 + ((1.0*x-center)/max(tiny, sigma))**2))
+            / max(tiny, (pi*sigma)))
+
+
+def split_lorentzian(x, amplitude=1.0, center=0.0, sigma=1.0, sigma_r=1.0):
+    """Return a 1-dimensional piecewise Lorentzian function.
+
+    Split means that width of the function is different between
+    left and right slope of the function. The peak height is calculated
+    from the condition that the integral from ``-.inf`` to ``+.inf`` is equal
+    to ``amplitude``.
+
+    split_lorentzian(x, amplitude, center, sigma, sigma_r) =
+        [2*amplitude / (pi* (sigma + sigma_r)] *
+         {  sigma**2   * (x<center)  / [sigma**2  + (x - center)**2]
+          + sigma_r**2 * (x>=center) / [sigma_r**2+ (x - center)**2] }
+
+    """
+    s = max(tiny, sigma)
+    r = max(tiny, sigma_r)
+    s2 = s*s
+    r2 = r*r
+    xc2 = (x-center)**2
+    amp = 2*amplitude/(pi*(s+r))
+    return amp*(s2*(x < center)/(s2+xc2) + r2*(x >= center)/(r2+xc2))
 
 
 def voigt(x, amplitude=1.0, center=0.0, sigma=1.0, gamma=None):
@@ -51,15 +76,15 @@ def voigt(x, amplitude=1.0, center=0.0, sigma=1.0, gamma=None):
     """
     if gamma is None:
         gamma = sigma
-    z = (x-center + 1j*gamma) / (sigma*s2)
-    return amplitude*wofz(z).real / (sigma*s2pi)
+    z = (x-center + 1j*gamma) / max(tiny, (sigma*s2))
+    return amplitude*wofz(z).real / max(tiny, (sigma*s2pi))
 
 
 def pvoigt(x, amplitude=1.0, center=0.0, sigma=1.0, fraction=0.5):
     """Return a 1-dimensional pseudo-Voigt function.
 
     pvoigt(x, amplitude, center, sigma, fraction) =
-       amplitude*(1-fraction)*gaussion(x, center, sigma_g) +
+       amplitude*(1-fraction)*gaussian(x, center, sigma_g) +
        amplitude*fraction*lorentzian(x, center, sigma)
 
     where sigma_g (the sigma for the Gaussian component) is
@@ -82,7 +107,7 @@ def moffat(x, amplitude=1, center=0., sigma=1, beta=1.):
         amplitude / (((x - center)/sigma)**2 + 1)**beta
 
     """
-    return amplitude / (((x - center)/sigma)**2 + 1)**beta
+    return amplitude / (((x - center)/max(tiny, sigma))**2 + 1)**beta
 
 
 def pearson7(x, amplitude=1.0, center=0.0, sigma=1.0, expon=1.0):
@@ -96,9 +121,10 @@ def pearson7(x, amplitude=1.0, center=0.0, sigma=1.0, expon=1.0):
     and beta() is the beta function.
 
     """
-    arg = (x-center)/sigma
+    expon = max(tiny, expon)
+    arg = (x-center)/max(tiny, sigma)
     scale = amplitude * gamfcn(expon)/(gamfcn(0.5)*gamfcn(expon-0.5))
-    return scale*(1+arg**2)**(-expon)/sigma
+    return scale*(1+arg**2)**(-expon)/max(tiny, sigma)
 
 
 def breit_wigner(x, amplitude=1.0, center=0.0, sigma=1.0, q=1.0):
@@ -136,11 +162,13 @@ def dho(x, amplitude=1., center=0., sigma=1., gamma=1.0):
         lp(x, center, sigma) = 1.0 / ((x+center)**2 + sigma**2)
 
     """
-    bose = (1.0 - exp(-x/gamma))
+    bose = (1.0 - exp(-x/max(tiny, gamma)))
     if isinstance(bose, (int, float)):
         bose = max(tiny, bose)
     else:
+        bose[where(isnan(bose))] = tiny
         bose[where(bose <= tiny)] = tiny
+
     lm = 1.0/((x-center)**2 + sigma**2)
     lp = 1.0/((x+center)**2 + sigma**2)
     return amplitude*sigma/pi*(lm - lp)/bose
@@ -153,7 +181,7 @@ def logistic(x, amplitude=1., center=0., sigma=1.):
         = amplitude*(1.  - 1. / (1 + exp((x-center)/sigma)))
 
     """
-    return amplitude*(1. - 1./(1. + exp((x-center)/sigma)))
+    return amplitude*(1. - 1./(1. + exp((x-center)/max(tiny, sigma))))
 
 
 def lognormal(x, amplitude=1.0, center=0., sigma=1):
@@ -167,7 +195,8 @@ def lognormal(x, amplitude=1.0, center=0., sigma=1):
         x = max(tiny, x)
     else:
         x[where(x <= tiny)] = tiny
-    return (amplitude/(x*sigma*s2pi)) * exp(-(log(x)-center)**2 / (2*sigma**2))
+    return ((amplitude/(x*max(tiny, sigma*s2pi))) *
+            exp(-(log(x)-center)**2 / max(tiny, (2*sigma**2))))
 
 
 def students_t(x, amplitude=1.0, center=0.0, sigma=1.0):
@@ -175,21 +204,22 @@ def students_t(x, amplitude=1.0, center=0.0, sigma=1.0):
 
     students_t(x, amplitude, center, sigma) =
 
-        gamma((sigma+1)/2)   (1 + (x-center)**2/sigma)^(-(sigma+1)/2)
-        -------------------------
+             gamma((sigma+1)/2)
+        ---------------------------- * (1 + (x-center)**2/sigma)^(-(sigma+1)/2)
         sqrt(sigma*pi)gamma(sigma/2)
 
     """
     s1 = (sigma+1)/2.0
-    denom = (sqrt(sigma*pi)*gamfcn(sigma/2))
-    return amplitude*(1 + (x-center)**2/sigma)**(-s1) * gamfcn(s1) / denom
+    denom = max(tiny, (sqrt(sigma*pi)*gamfcn(max(tiny, sigma/2))))
+    return amplitude*(1 + (x-center)**2/max(tiny, sigma))**(-s1) * gamfcn(s1) / denom
 
 
 def expgaussian(x, amplitude=1, center=0, sigma=1.0, gamma=1.0):
-    """Return a exponentially modified Gaussian.
+    """Return an exponentially modified Gaussian.
 
-    expgaussian(x, amplitude, center, sigma, gamma=)
-        = (gamma/2) exp[center*gamma + (gamma*sigma)**2/2 - gamma*x] *
+    expgaussian(x, amplitude, center, sigma, gamma)
+        = amplitude * (gamma/2) *
+          exp[center*gamma + (gamma*sigma)**2/2 - gamma*x] *
           erfc[(center + gamma*sigma**2 - x)/(sqrt(2)*sigma)]
 
     https://en.wikipedia.org/wiki/Exponentially_modified_Gaussian_distribution
@@ -197,14 +227,14 @@ def expgaussian(x, amplitude=1, center=0, sigma=1.0, gamma=1.0):
     """
     gss = gamma*sigma*sigma
     arg1 = gamma*(center + gss/2.0 - x)
-    arg2 = (center + gss - x)/(s2*sigma)
+    arg2 = (center + gss - x)/max(tiny, (s2*sigma))
     return amplitude*(gamma/2) * exp(arg1) * erfc(arg2)
 
 
-def donaich(x, amplitude=1.0, center=0, sigma=1.0, gamma=0.0):
+def doniach(x, amplitude=1.0, center=0, sigma=1.0, gamma=0.0):
     """Return a Doniach Sunjic asymmetric lineshape, used for photo-emission.
 
-    donaich(x, amplitude, center, sigma, gamma) =
+    doniach(x, amplitude, center, sigma, gamma) =
         amplitude / sigma^(1-gamma) *
         cos(pi*gamma/2 + (1-gamma) arctan((x-center)/sigma) /
         (sigma**2 + (x-center)**2)**[(1-gamma)/2]
@@ -212,12 +242,15 @@ def donaich(x, amplitude=1.0, center=0, sigma=1.0, gamma=0.0):
     see http://www.casaxps.com/help_manual/line_shapes.htm
 
     """
-    arg = (x-center)/sigma
+    arg = (x-center)/max(tiny, sigma)
     gm1 = (1.0 - gamma)
-    scale = amplitude/(sigma**gm1)
+    scale = amplitude/max(tiny, (sigma**gm1))
     return scale*cos(pi*gamma/2 + gm1*arctan(arg))/(1 + arg**2)**(gm1/2)
 
 
+donaich = doniach   # for back-compat
+
+
 def skewed_gaussian(x, amplitude=1.0, center=0.0, sigma=1.0, gamma=0.0):
     """Return a Gaussian lineshape, skewed with error function.
 
@@ -231,7 +264,7 @@ def skewed_gaussian(x, amplitude=1.0, center=0.0, sigma=1.0, gamma=0.0):
     see https://en.wikipedia.org/wiki/Skew_normal_distribution
 
     """
-    asym = 1 + erf(gamma*(x-center)/(s2*sigma))
+    asym = 1 + erf(gamma*(x-center)/max(tiny, (s2*sigma)))
     return asym * gaussian(x, amplitude, center, sigma)
 
 
@@ -249,7 +282,7 @@ def skewed_voigt(x, amplitude=1.0, center=0.0, sigma=1.0, gamma=None, skew=0.0):
     see https://en.wikipedia.org/wiki/Skew_normal_distribution
 
     """
-    beta = skew/(s2*sigma)
+    beta = skew/max(tiny, (s2*sigma))
     asym = 1 + erf(beta*(x-center))
     return asym * voigt(x, amplitude, center, sigma, gamma=gamma)
 
@@ -274,6 +307,43 @@ def expsine(x, amplitude=1.0, frequency=1.0, shift=0.0, decay=0.0):
     return amplitude*sin(x*frequency + shift) * exp(-x*decay)
 
 
+def thermal_distribution(x, amplitude=1.0, center=0.0, kt=1.0, form='bose'):
+    """Return a thermal distribution function.
+
+    form = 'bose' (default) is the Bose-Einstein distribution
+    thermal_distribution(x, amplitude=1.0, center=0.0, kt=1.0):
+       = 1/(amplitude*exp((x - center)/kt) - 1)
+    form = 'maxwell' is the Maxwell-Boltzmann distribution
+    thermal_distribution(x, amplitude=1.0, center=0.0, kt=1.0):
+       = 1/(amplitude*exp((x - center)/kt))
+    form = 'fermi' is the Fermi-Dirac distribution
+    thermal_distribution(x, amplitude=1.0, center=0.0, kt=1.0):
+       = 1/(amplitude*exp((x - center)/kt) + 1)
+
+    Notes:
+    - ``kt`` should be defined in the same units as ``x``. (The Boltzmann
+      constant is kB = 8.617e-5 eV/K).
+    - set ``kt<0`` to implement the energy loss convention common in scattering
+      research.
+
+    see http://hyperphysics.phy-astr.gsu.edu/hbase/quantum/disfcn.html
+
+    """
+    form = form.lower()
+    if form.startswith('bose'):
+        offset = -1
+    elif form.startswith('maxwell'):
+        offset = 0
+    elif form.startswith('fermi'):
+        offset = 1
+    else:
+        msg = "Invalid value ('%s') for argument 'form'; should be one of %s."\
+              % (form, "'maxwell', 'fermi', or 'bose'")
+        raise ValueError(msg)
+
+    return real(1/(amplitude*exp((x - center)/kt) + offset + tiny*1j))
+
+
 def step(x, amplitude=1.0, center=0.0, sigma=1.0, form='linear'):
     """Return a step function.
 
@@ -287,19 +357,22 @@ def step(x, amplitude=1.0, center=0.0, sigma=1.0, form='linear'):
     where arg = (x - center)/sigma
 
     """
-    if abs(sigma) < tiny:
-        sigma = tiny
+    out = (x - center)/max(tiny, sigma)
 
-    out = (x - center)/sigma
     if form == 'erf':
         out = 0.5*(1 + erf(out))
-    elif form.startswith('logi'):
+    elif form == 'logistic':
         out = (1. - 1./(1. + exp(out)))
     elif form in ('atan', 'arctan'):
         out = 0.5 + arctan(out)/pi
-    else:
+    elif form == 'linear':
         out[where(out < 0)] = 0.0
         out[where(out > 1)] = 1.0
+    else:
+        msg = "Invalid value ('%s') for argument 'form'; should be one of %s."\
+               % (form, "'erf', 'logistic', 'atan', 'arctan', or 'linear'")
+        raise ValueError(msg)
+
     return amplitude*out
 
 
@@ -319,71 +392,36 @@ def rectangle(x, amplitude=1.0, center1=0.0, sigma1=1.0,
     and   arg2 = -(x - center2)/sigma2
 
     """
-    if abs(sigma1) < tiny:
-        sigma1 = tiny
-    if abs(sigma2) < tiny:
-        sigma2 = tiny
+    arg1 = (x - center1)/max(tiny, sigma1)
+    arg2 = (center2 - x)/max(tiny, sigma2)
 
-    arg1 = (x - center1)/sigma1
-    arg2 = (center2 - x)/sigma2
     if form == 'erf':
         out = 0.5*(erf(arg1) + erf(arg2))
-    elif form.startswith('logi'):
+    elif form == 'logistic':
         out = (1. - 1./(1. + exp(arg1)) - 1./(1. + exp(arg2)))
     elif form in ('atan', 'arctan'):
         out = (arctan(arg1) + arctan(arg2))/pi
-    else:
+    elif form == 'linear':
         arg1[where(arg1 < 0)] = 0.0
         arg1[where(arg1 > 1)] = 1.0
         arg2[where(arg2 > 0)] = 0.0
         arg2[where(arg2 < -1)] = -1.0
         out = arg1 + arg2
+    else:
+        msg = "Invalid value ('%s') for argument 'form'; should be one of %s."\
+               % (form, "'erf', 'logistic', 'atan', 'arctan', or 'linear'")
+        raise ValueError(msg)
+
     return amplitude*out
 
 
-def _erf(x):
-    """Return the error function.
-
-    erf = 2/sqrt(pi)*integral(exp(-t**2), t=[0, z])
-
-    """
-    return erf(x)
-
-
-def _erfc(x):
-    """Return the complementary error function.
-
-    erfc = 1 - erf(x)
-
-    """
-    return erfc(x)
-
-
-def _wofz(x):
-    """Return the fadeeva function for complex argument.
-
-    wofz = exp(-x**2)*erfc(-i*x)
-
-    """
-    return wofz(x)
-
-
-def _gamma(x):
-    """Return the gamma function."""
-    return gamfcn(x)
-
-
-def _gammaln(x):
-    """Return the log of absolute value of gamma function."""
-    return gammaln(x)
-
-
 def exponential(x, amplitude=1, decay=1):
     """Return an exponential function.
 
     x -> amplitude * exp(-x/decay)
 
     """
+    decay = max(tiny, decay)
     return amplitude * exp(-x/decay)
 
 
@@ -412,11 +450,3 @@ def parabolic(x, a=0.0, b=0.0, c=0.0):
 
     """
     return a * x**2 + b * x + c
-
-
-def assert_results_close(actual, desired, rtol=1e-03, atol=1e-03,
-                         err_msg='', verbose=True):
-    """Check whether all actual and desired parameter values are close."""
-    for param_name, value in desired.items():
-        assert_allclose(actual[param_name], value, rtol,
-                        atol, err_msg, verbose)
