@@ -1,14 +1,15 @@
 """Parameter class."""
-from __future__ import division
 
 from collections import OrderedDict
 from copy import deepcopy
 import json
 
 from asteval import Interpreter, get_ast_names, valid_symbol_name
-from numpy import arcsin, array, cos, inf, nan, sin, sqrt
+from numpy import arcsin, array, cos, inf, isclose, sin, sqrt
 import scipy.special
-import uncertainties
+
+from .jsonutils import decode4js, encode4js
+from .printfuncs import params_html_table
 
 SCIPY_FUNCTIONS = {'gamfcn': scipy.special.gamma}
 for name in ('erf', 'erfc', 'wofz'):
@@ -21,31 +22,6 @@ def check_ast_errors(expr_eval):
         expr_eval.raise_exception(None)
 
 
-def isclose(x, y, rtol=1e-5, atol=1e-8):
-    """Check whether two numbers are the same within a tolerance.
-
-    abs(`x` - `y`) <= (`atol` + `rtol` * abs(`y`))
-
-    Parameters
-    ----------
-    x, y : float
-        Input values.
-    rtol : float, optional
-        The relative tolerance parameter.
-    atol : float, optional
-        The absolute tolerance parameter.
-
-    Returns
-    -------
-    bool
-        True if `x` and `y` are the same within tolerance, otherwise False.
-
-    """
-    if x in (-inf, inf, nan) or y in (-inf, inf, nan):
-        return (x == y)
-    return (abs(x - y) <= atol + rtol * abs(y))
-
-
 class Parameters(OrderedDict):
     """An ordered dictionary of all the Parameter objects required to
     specify a fit model. All minimization and Model fitting routines in
@@ -66,37 +42,59 @@ class Parameters(OrderedDict):
 
     """
 
-    def __init__(self, asteval=None, *args, **kwds):
+    def __init__(self, asteval=None, usersyms=None, *args, **kwds):
         """
         Arguments
         ---------
         asteval : :class:`asteval.Interpreter`, optional
             Instance of the asteval Interpreter to use for constraint
             expressions. If None, a new interpreter will be created.
+            Warning: *deprecated, use usersyms if possible*
+        usersyms : dictionary of symbols to add to the
+            :class:`asteval.Interpreter`.
         *args : optional
             Arguments.
         **kwds : optional
             Keyword arguments.
 
         """
-        super(Parameters, self).__init__(self)
-        self._asteval = asteval
+        super().__init__(self)
 
-        if asteval is None:
-            self._asteval = Interpreter(usersyms=SCIPY_FUNCTIONS)
-        self.update(*args, **kwds)
+        self._asteval = asteval
+        if self._asteval is None:
+            self._asteval = Interpreter()
+
+        _syms = {}
+        _syms.update(SCIPY_FUNCTIONS)
+        if usersyms is not None:
+            _syms.update(usersyms)
+        for key, val in _syms.items():
+            self._asteval.symtable[key] = val
 
     def copy(self):
         """Parameters.copy() should always be a deepcopy."""
         return self.__deepcopy__(None)
 
+    def update(self, other):
+        """Update values and symbols with another Parameters object."""
+        if not isinstance(other, Parameters):
+            raise ValueError("'%s' is not a Parameters object" % other)
+        self.add_many(*other.values())
+        for sym in other._asteval.user_defined_symbols():
+            self._asteval.symtable[sym] = other._asteval.symtable[sym]
+        return self
+
     def __copy__(self):
         """Parameters.copy() should always be a deepcopy."""
         return self.__deepcopy__(None)
 
     def __deepcopy__(self, memo):
-        """Parameters.deepcopy() needs to make sure that asteval is available
-        and that all individual Parameter objects are copied."""
+        """Implementation of Parameters.deepcopy().
+
+        The method needs to make sure that asteval is available and that all
+        individual Parameter objects are copied.
+
+        """
         _pars = Parameters(asteval=None)
 
         # find the symbols that were added by users, not during construction
@@ -126,7 +124,7 @@ class Parameters(OrderedDict):
         return _pars
 
     def __setitem__(self, key, par):
-        """TODO: add magic method docstring."""
+        """Set items of Parameters object."""
         if key not in self:
             if not valid_symbol_name(key):
                 raise KeyError("'%s' is not a valid Parameters name" % key)
@@ -142,16 +140,15 @@ class Parameters(OrderedDict):
         if not isinstance(other, Parameters):
             raise ValueError("'%s' is not a Parameters object" % other)
         out = deepcopy(self)
-        params = other.values()
-        out.add_many(*params)
+        out.add_many(*other.values())
+        for sym in other._asteval.user_defined_symbols():
+            if sym not in out._asteval.symtable:
+                out._asteval.symtable[sym] = other._asteval.symtable[sym]
         return out
 
     def __iadd__(self, other):
         """Add/assign Parameters objects."""
-        if not isinstance(other, Parameters):
-            raise ValueError("'%s' is not a Parameters object" % other)
-        params = other.values()
-        self.add_many(*params)
+        self.update(other)
         return self
 
     def __array__(self):
@@ -190,16 +187,36 @@ class Parameters(OrderedDict):
         # an Error. Another way of doing this would be to remove all the expr
         # from the Parameter instances before they get added, then to restore
         # them.
-        self._asteval.symtable.update(state['unique_symbols'])
+
+        symtab = self._asteval.symtable
+        for key, val in state['unique_symbols'].items():
+            if key not in symtab:
+                symtab[key] = val
 
         # then add all the parameters
         self.add_many(*state['params'])
 
+    def eval(self, expr):
+        """Evaluate a statement using the asteval Interpreter.
+
+        Parameters
+        ----------
+        expr : string
+            An expression containing parameter names and other symbols
+            recognizable by the asteval Interpreter.
+
+        Returns
+        -------
+           The result of the expression.
+
+        """
+        return self._asteval.eval(expr)
+
     def update_constraints(self):
         """Update all constrained parameters, checking that dependencies are
         evaluated as needed."""
-        requires_update = set(name for name, par in self.items()
-                              if par._expr is not None)
+        requires_update = {name for name, par in self.items() if par._expr is
+                           not None}
         updated_tracker = set(requires_update)
 
         def _update_param(name):
@@ -238,7 +255,7 @@ class Parameters(OrderedDict):
 
         """
         if oneline:
-            return super(Parameters, self).__repr__()
+            return super().__repr__()
         s = "Parameters({\n"
         for key in self.keys():
             s += "    '%s': %s, \n" % (key, self[key])
@@ -292,6 +309,10 @@ class Parameters(OrderedDict):
             print(line.format(name_len=name_len, n=colwidth, p=precision,
                               f=fmt, **pvalues))
 
+    def _repr_html_(self):
+        """Returns a HTML representation of parameters data."""
+        return params_html_table(self)
+
     def add(self, name, value=None, vary=True, min=-inf, max=inf, expr=None,
             brute_step=None):
         """Add a Parameter.
@@ -359,12 +380,16 @@ class Parameters(OrderedDict):
         >>> params.add_many(f, g)
 
         """
-        for para in parlist:
-            if isinstance(para, Parameter):
-                self.__setitem__(para.name, para)
-            else:
-                param = Parameter(*para)
-                self.__setitem__(param.name, param)
+        __params = []
+        for par in parlist:
+            if not isinstance(par, Parameter):
+                par = Parameter(*par)
+            __params.append(par)
+            par._delay_asteval = True
+            self.__setitem__(par.name, par)
+
+        for para in __params:
+            para._delay_asteval = False
 
     def valuesdict(self):
         """Return an ordered dictionary of parameter values.
@@ -376,7 +401,7 @@ class Parameters(OrderedDict):
            Parameter.
 
         """
-        return OrderedDict(((p.name, p.value) for p in self.values()))
+        return OrderedDict((p.name, p.value) for p in self.values())
 
     def dumps(self, **kws):
         """Represent Parameters as a JSON string.
@@ -398,7 +423,7 @@ class Parameters(OrderedDict):
         """
         params = [p.__getstate__() for p in self.values()]
         sym_unique = self._asteval.user_defined_symbols()
-        unique_symbols = {key: deepcopy(self._asteval.symtable[key])
+        unique_symbols = {key: encode4js(deepcopy(self._asteval.symtable[key]))
                           for key in sym_unique}
         return json.dumps({'unique_symbols': unique_symbols,
                            'params': params}, **kws)
@@ -429,10 +454,12 @@ class Parameters(OrderedDict):
         self.clear()
 
         tmp = json.loads(s, **kws)
-        state = {'unique_symbols': tmp['unique_symbols'],
-                 'params': []}
+        unique_symbols = {key: decode4js(tmp['unique_symbols'][key]) for key
+                          in tmp['unique_symbols']}
+
+        state = {'unique_symbols': unique_symbols, 'params': []}
         for parstate in tmp['params']:
-            _par = Parameter()
+            _par = Parameter(name='')
             _par.__setstate__(parstate)
             state['params'].append(_par)
         self.__setstate__(state)
@@ -450,9 +477,8 @@ class Parameters(OrderedDict):
 
         Returns
         -------
-        None or int
-            Return value from `fp.write()`. None for Python 2.7 and the
-            number of characters written in Python 3.
+        int
+            Return value from `fp.write()`: the number of characters written.
 
         See Also
         --------
@@ -484,7 +510,7 @@ class Parameters(OrderedDict):
         return self.loads(fp.read(), **kws)
 
 
-class Parameter(object):
+class Parameter:
     """A Parameter is an object that can be varied in a fit, or one of the
     controlling variables in a model. It is a central component of lmfit,
     and all minimization and modeling methods use Parameter objects.
@@ -495,7 +521,7 @@ class Parameter(object):
     placed on the Parameter's value by setting its `min` and/or `max`
     attributes.  A Parameter can also have its value determined by a
     mathematical expression of other Parameter values held in the `expr`
-    attrribute.  Additional attributes include `brute_step` used as the step
+    attribute.  Additional attributes include `brute_step` used as the step
     size in a brute-force minimization, and `user_data` reserved
     exclusively for user's need.
 
@@ -506,12 +532,12 @@ class Parameter(object):
 
     """
 
-    def __init__(self, name=None, value=None, vary=True, min=-inf, max=inf,
+    def __init__(self, name, value=None, vary=True, min=-inf, max=inf,
                  expr=None, brute_step=None, user_data=None):
         """
         Parameters
         ----------
-        name : str, optional
+        name : str
             Name of the Parameter.
         value : float, optional
             Numerical Parameter value.
@@ -540,7 +566,6 @@ class Parameter(object):
 
         """
         self.name = name
-        self._val = value
         self.user_data = user_data
         self.init_value = value
         self.min = min
@@ -555,6 +580,7 @@ class Parameter(object):
         self.stderr = None
         self.correl = None
         self.from_internal = lambda val: val
+        self._val = value
         self._init_bounds()
 
     def set(self, value=None, vary=None, min=None, max=None, expr=None,
@@ -605,10 +631,6 @@ class Parameter(object):
             par.set(brute_step=0)     # removes brute_step
 
         """
-        if value is not None:
-            self.value = value
-            self.__set_expression('')
-
         if vary is not None:
             self.vary = vary
             if vary:
@@ -620,6 +642,12 @@ class Parameter(object):
         if max is not None:
             self.max = max
 
+        # need to set this after min and max, so that it will use new
+        # bounds in the setter for value
+        if value is not None:
+            self.value = value
+            self.__set_expression("")
+
         if expr is not None:
             self.__set_expression(expr)
 
@@ -656,32 +684,31 @@ class Parameter(object):
 
     def __setstate__(self, state):
         """Set state for pickle."""
-        (self.name, self.value, self.vary, self.expr, self.min, self.max,
+        (self.name, _value, self.vary, self.expr, self.min, self.max,
          self.brute_step, self.stderr, self.correl, self.init_value,
          self.user_data) = state
         self._expr_ast = None
         self._expr_eval = None
         self._expr_deps = []
         self._delay_asteval = False
+        self.value = _value
         self._init_bounds()
 
     def __repr__(self):
-        """Returns printable representation of a Parameter object."""
+        """Return printable representation of a Parameter object."""
         s = []
-        if self.name is not None:
-            s.append("'%s'" % self.name)
-        sval = repr(self._getval())
+        sval = "value=%s" % repr(self._getval())
         if not self.vary and self._expr is None:
-            sval = "value=%s (fixed)" % sval
+            sval += " (fixed)"
         elif self.stderr is not None:
-            sval = "value=%s +/- %.3g" % (sval, self.stderr)
+            sval += " +/- %.3g" % self.stderr
         s.append(sval)
         s.append("bounds=[%s:%s]" % (repr(self.min), repr(self.max)))
         if self._expr is not None:
             s.append("expr='%s'" % self.expr)
         if self.brute_step is not None:
             s.append("brute_step=%s" % (self.brute_step))
-        return "<Parameter %s>" % ', '.join(s)
+        return "<Parameter '%s', %s>" % (self.name, ', '.join(s))
 
     def setup_bounds(self):
         """Set up Minuit-style internal/external parameter transformation of
@@ -749,42 +776,18 @@ class Parameter(object):
         """Get value, with bounds applied."""
         # Note assignment to self._val has been changed to self.value
         # The self.value property setter makes sure that the
-        # _expr_eval.symtable is kept updated.
-        # If you just assign to self._val then
-        # _expr_eval.symtable[self.name]
+        # _expr_eval.symtable is kept up-to-date.
+        # If you just assign to self._val then _expr_eval.symtable[self.name]
         # becomes stale if parameter.expr is not None.
-        if (isinstance(self._val, uncertainties.core.Variable) and
-                self._val is not nan):
-
-            try:
-                self.value = self._val.nominal_value
-            except AttributeError:
-                pass
-        if not self.vary and self._expr is None:
-            return self._val
-
         if self._expr is not None:
             if self._expr_ast is None:
                 self.__set_expression(self._expr)
-
             if self._expr_eval is not None:
                 if not self._delay_asteval:
                     self.value = self._expr_eval(self._expr_ast)
                     check_ast_errors(self._expr_eval)
-
-        if self._val is not None:
-            if self._val > self.max:
-                self._val = self.max
-            elif self._val < self.min:
-                self._val = self.min
-        if self._expr_eval is not None:
-            self._expr_eval.symtable[self.name] = self._val
         return self._val
 
-    def set_expr_eval(self, evaluator):
-        """Set expression evaluator instance."""
-        self._expr_eval = evaluator
-
     @property
     def value(self):
         """Return the numerical value of the Parameter, with bounds applied."""
@@ -794,10 +797,15 @@ class Parameter(object):
     def value(self, val):
         """Set the numerical Parameter value."""
         self._val = val
+        if self._val is not None:
+            if self._val > self.max:
+                self._val = self.max
+            elif self._val < self.min:
+                self._val = self.min
         if not hasattr(self, '_expr_eval'):
             self._expr_eval = None
         if self._expr_eval is not None:
-            self._expr_eval.symtable[self.name] = val
+            self._expr_eval.symtable[self.name] = self._val
 
     @property
     def expr(self):
@@ -826,6 +834,8 @@ class Parameter(object):
         if val is None:
             self._expr_ast = None
         if val is not None and self._expr_eval is not None:
+            self._expr_eval.error = []
+            self._expr_eval.error_msg = None
             self._expr_ast = self._expr_eval.parse(val)
             check_ast_errors(self._expr_eval)
             self._expr_deps = get_ast_names(self._expr_ast)
@@ -850,8 +860,8 @@ class Parameter(object):
         """positive"""
         return +self._getval()
 
-    def __nonzero__(self):
-        """not zero"""
+    def __bool__(self):
+        """bool"""
         return self._getval() != 0
 
     def __int__(self):
@@ -874,10 +884,9 @@ class Parameter(object):
         """-"""
         return self._getval() - other
 
-    def __div__(self, other):
+    def __truediv__(self, other):
         """/"""
         return self._getval() / other
-    __truediv__ = __div__
 
     def __floordiv__(self, other):
         """//"""
@@ -927,10 +936,9 @@ class Parameter(object):
         """+ (right)"""
         return other + self._getval()
 
-    def __rdiv__(self, other):
+    def __rtruediv__(self, other):
         """/ (right)"""
         return other / self._getval()
-    __rtruediv__ = __rdiv__
 
     def __rdivmod__(self, other):
         """divmod (right)"""
@@ -955,9 +963,3 @@ class Parameter(object):
     def __rsub__(self, other):
         """- (right)"""
         return other - self._getval()
-
-
-def isParameter(x):
-    """Test for Parameter-ness."""
-    return (isinstance(x, Parameter) or
-            x.__class__.__name__ == 'Parameter')
