@@ -1,34 +1,39 @@
-# -*- coding: utf-8 -*-
-from __future__ import print_function
-from lmfit import minimize, Parameters, Parameter, report_fit, Minimizer
-from lmfit.minimizer import (SCALAR_METHODS, HAS_EMCEE,
-                             MinimizerResult, _lnpost, _nan_policy)
-from lmfit.lineshapes import gaussian
-from lmfit import ufloat
+import unittest
+
 import numpy as np
 from numpy import pi
-from numpy.testing import (assert_, decorators, assert_raises,
-                           assert_almost_equal, assert_equal,
-                           assert_allclose)
-import unittest
-import nose
-from nose import SkipTest
+from numpy.testing import (assert_allclose, assert_almost_equal, assert_equal,
+                           dec)
+import pytest
+from uncertainties import ufloat
+
+from lmfit import Minimizer, Parameters, minimize
+from lmfit.lineshapes import gaussian
+from lmfit.minimizer import (HAS_EMCEE, SCALAR_METHODS, MinimizerResult,
+                             _nan_policy)
+
+try:
+    import numdifftools  # noqa: F401
+    HAS_NUMDIFFTOOLS = True
+except ImportError:
+    HAS_NUMDIFFTOOLS = False
 
 
 def check(para, real_val, sig=3):
     err = abs(para.value - real_val)
-    print('Check Param w/ stderr: ',  para.name, para.value, real_val, para.stderr)
-    assert(err < sig * para.stderr)
+    assert err < sig * para.stderr
+
 
 def check_wo_stderr(para, real_val, sig=0.1):
     err = abs(para.value - real_val)
-    print('Check Param w/o stderr: ', para.name, para.value, real_val, sig)
-    assert(err < sig)
+    assert err < sig
+
 
 def check_paras(para_fit, para_real, sig=3):
     for i in para_fit:
         check(para_fit[i], para_real[i].value, sig=sig)
 
+
 def test_simple():
     # create data to be fitted
     np.random.seed(1)
@@ -38,7 +43,7 @@ def test_simple():
 
     # define objective function: returns the array to be minimized
     def fcn2min(params, x, data):
-        """ model decaying sine wave, subtract data"""
+        """model decaying sine wave, subtract data"""
         amp = params['amp']
         shift = params['shift']
         omega = params['omega']
@@ -49,28 +54,19 @@ def test_simple():
 
     # create a set of Parameters
     params = Parameters()
-    params.add('amp',   value= 10,  min=0)
-    params.add('decay', value= 0.1)
-    params.add('shift', value= 0.0, min=-pi / 2., max=pi / 2)
-    params.add('omega', value= 3.0)
+    params.add('amp', value=10, min=0)
+    params.add('decay', value=0.1)
+    params.add('shift', value=0.0, min=-pi/2., max=pi/2)
+    params.add('omega', value=3.0)
 
     # do fit, here with leastsq model
     result = minimize(fcn2min, params, args=(x, data))
 
-    # calculate final result
-    final = data + result.residual
-
-    # write error report
-    print(" --> SIMPLE --> ")
-    print(result.params)
-    report_fit(result.params)
-
-    #assert that the real parameters are found
-
+    # assert that the real parameters are found
     for para, val in zip(result.params.values(), [5, 0.025, -.1, 2]):
-
         check(para, val)
 
+
 def test_lbfgsb():
     p_true = Parameters()
     p_true.add('amp', value=14.0)
@@ -89,37 +85,31 @@ def test_lbfgsb():
         model = amp * np.sin(shift + x / per) * np.exp(-x * x * decay * decay)
         if data is None:
             return model
-        return (model - data)
+        return model - data
 
     n = 2500
     xmin = 0.
     xmax = 250.0
     noise = np.random.normal(scale=0.7215, size=n)
-    x     = np.linspace(xmin, xmax, n)
-    data  = residual(p_true, x) + noise
+    x = np.linspace(xmin, xmax, n)
+    data = residual(p_true, x) + noise
 
     fit_params = Parameters()
     fit_params.add('amp', value=11.0, min=5, max=20)
     fit_params.add('period', value=5., min=1., max=7)
-    fit_params.add('shift', value=.10,  min=0.0, max=0.2)
+    fit_params.add('shift', value=.10, min=0.0, max=0.2)
     fit_params.add('decay', value=6.e-3, min=0, max=0.1)
 
-    init = residual(fit_params, x)
-
-    out = minimize(residual, fit_params, method='lbfgsb', args=(x,), kws={'data':data})
-
-    fit = residual(fit_params, x)
-
-    for name, par in out.params.items():
-        nout = "%s:%s" % (name, ' '*(20-len(name)))
-        print("%s: %s (%s) " % (nout, par.value, p_true[name].value))
+    out = minimize(residual, fit_params, method='lbfgsb', args=(x,),
+                   kws={'data': data})
 
     for para, true_para in zip(out.params.values(), p_true.values()):
         check_wo_stderr(para, true_para.value)
 
+
 def test_derive():
     def func(pars, x, data=None):
-        model= pars['a'] * np.exp(-pars['b'] * x) + pars['c']
+        model = pars['a'] * np.exp(-pars['b'] * x) + pars['c']
         if data is None:
             return model
         return model - data
@@ -129,7 +119,7 @@ def test_derive():
         return np.array([v, -pars['a']*x*v, np.ones(len(x))])
 
     def f(var, x):
-        return var[0]* np.exp(-var[1] * x)+var[2]
+        return var[0] * np.exp(-var[1] * x) + var[2]
 
     params1 = Parameters()
     params1.add('a', value=10)
@@ -142,44 +132,23 @@ def test_derive():
     params2.add('c', value=10)
 
     a, b, c = 2.5, 1.3, 0.8
-    x = np.linspace(0,4,50)
+    x = np.linspace(0, 4, 50)
     y = f([a, b, c], x)
     data = y + 0.15*np.random.normal(size=len(x))
 
     # fit without analytic derivative
-    min1 = Minimizer(func, params1, fcn_args=(x,), fcn_kws={'data':data})
+    min1 = Minimizer(func, params1, fcn_args=(x,), fcn_kws={'data': data})
     out1 = min1.leastsq()
-    fit1 = func(out1.params, x)
 
     # fit with analytic derivative
-    min2 = Minimizer(func, params2, fcn_args=(x,), fcn_kws={'data':data})
+    min2 = Minimizer(func, params2, fcn_args=(x,), fcn_kws={'data': data})
     out2 = min2.leastsq(Dfun=dfunc, col_deriv=1)
-    fit2 = func(out2.params, x)
-
-    print ('''Comparison of fit to exponential decay
-    with and without analytic derivatives, to
-       model = a*exp(-b*x) + c
-    for a = %.2f, b = %.2f, c = %.2f
-    ==============================================
-    Statistic/Parameter|   Without   | With      |
-    ----------------------------------------------
-    N Function Calls   |   %3i       |   %3i     |
-    Chi-square         |   %.4f    |   %.4f  |
-       a               |   %.4f    |   %.4f  |
-       b               |   %.4f    |   %.4f  |
-       c               |   %.4f    |   %.4f  |
-    ----------------------------------------------
-    ''' %  (a, b, c,
-            out1.nfev,   out2.nfev,
-            out1.chisqr, out2.chisqr,
-            out1.params['a'].value, out2.params['a'].value,
-            out1.params['b'].value, out2.params['b'].value,
-            out1.params['c'].value, out2.params['c'].value ))
 
     check_wo_stderr(out1.params['a'], out2.params['a'].value, 0.00005)
     check_wo_stderr(out1.params['b'], out2.params['b'].value, 0.00005)
     check_wo_stderr(out1.params['c'], out2.params['c'].value, 0.00005)
 
+
 def test_peakfit():
     def residual(pars, x, data=None):
         g1 = gaussian(x, pars['a1'], pars['c1'], pars['w1'])
@@ -187,9 +156,9 @@ def test_peakfit():
         model = g1 + g2
         if data is None:
             return model
-        return (model - data)
+        return model - data
 
-    n    = 601
+    n = 601
     xmin = 0.
     xmax = 15.0
     noise = np.random.normal(scale=.65, size=n)
@@ -197,40 +166,28 @@ def test_peakfit():
 
     org_params = Parameters()
     org_params.add_many(('a1', 12.0, True, None, None, None),
-                        ('c1',  5.3, True, None, None, None),
-                        ('w1',  1.0, True, None, None, None),
-                        ('a2',  9.1, True, None, None, None),
-                        ('c2',  8.1, True, None, None, None),
-                        ('w2',  2.5, True, None, None, None))
-
-    data  = residual(org_params, x) + noise
+                        ('c1', 5.3, True, None, None, None),
+                        ('w1', 1.0, True, None, None, None),
+                        ('a2', 9.1, True, None, None, None),
+                        ('c2', 8.1, True, None, None, None),
+                        ('w2', 2.5, True, None, None, None))
 
+    data = residual(org_params, x) + noise
 
     fit_params = Parameters()
-    fit_params.add_many(('a1',  8.0, True, None, 14., None),
-                        ('c1',  5.0, True, None, None, None),
-                        ('w1',  0.7, True, None, None, None),
-                        ('a2',  3.1, True, None, None, None),
-                        ('c2',  8.8, True, None, None, None))
+    fit_params.add_many(('a1', 8.0, True, None, 14., None),
+                        ('c1', 5.0, True, None, None, None),
+                        ('w1', 0.7, True, None, None, None),
+                        ('a2', 3.1, True, None, None, None),
+                        ('c2', 8.8, True, None, None, None))
 
     fit_params.add('w2', expr='2.5*w1')
 
-    myfit = Minimizer(residual, fit_params,
-                      fcn_args=(x,), fcn_kws={'data': data})
+    myfit = Minimizer(residual, fit_params, fcn_args=(x,),
+                      fcn_kws={'data': data})
 
     myfit.prepare_fit()
-
-    init = residual(fit_params, x)
-
-
     out = myfit.leastsq()
-
-    # print(' N fev = ', myfit.nfev)
-    # print(myfit.chisqr, myfit.redchi, myfit.nfree)
-
-    report_fit(out.params)
-
-    fit = residual(out.params, x)
     check_paras(out.params, org_params)
 
 
@@ -246,11 +203,11 @@ def test_scalar_minimize_has_no_uncertainties():
     np.random.seed(1)
     x = np.linspace(0, 15, 301)
     data = (5. * np.sin(2 * x - 0.1) * np.exp(-x*x*0.025) +
-            np.random.normal(size=len(x), scale=0.2) )
+            np.random.normal(size=len(x), scale=0.2))
 
     # define objective function: returns the array to be minimized
     def fcn2min(params, x, data):
-        """ model decaying sine wave, subtract data"""
+        """model decaying sine wave, subtract data"""
         amp = params['amp']
         shift = params['shift']
         omega = params['omega']
@@ -261,26 +218,22 @@ def test_scalar_minimize_has_no_uncertainties():
 
     # create a set of Parameters
     params = Parameters()
-    params.add('amp',   value= 10,  min=0)
-    params.add('decay', value= 0.1)
-    params.add('shift', value= 0.0, min=-pi / 2., max=pi / 2)
-    params.add('omega', value= 3.0)
+    params.add('amp', value=10, min=0)
+    params.add('decay', value=0.1)
+    params.add('shift', value=0.0, min=-pi/2., max=pi/2)
+    params.add('omega', value=3.0)
 
     mini = Minimizer(fcn2min, params, fcn_args=(x, data))
     out = mini.minimize()
-    assert_(np.isfinite(out.params['amp'].stderr))
-    print(out.errorbars)
-    assert_(out.errorbars == True)
+    assert np.isfinite(out.params['amp'].stderr)
+    assert out.errorbars
     out2 = mini.minimize(method='nelder-mead')
-    assert_(out2.params['amp'].stderr is None)
-    assert_(out2.params['decay'].stderr is None)
-    assert_(out2.params['shift'].stderr is None)
-    assert_(out2.params['omega'].stderr is None)
-    assert_(out2.params['amp'].correl is None)
-    assert_(out2.params['decay'].correl is None)
-    assert_(out2.params['shift'].correl is None)
-    assert_(out2.params['omega'].correl is None)
-    assert_(out2.errorbars == False)
+
+    for par in ('amp', 'decay', 'shift', 'omega'):
+        assert HAS_NUMDIFFTOOLS == (out2.params[par].stderr is not None)
+        assert HAS_NUMDIFFTOOLS == (out2.params[par].correl is not None)
+
+    assert HAS_NUMDIFFTOOLS == out2.errorbars
 
 
 def test_scalar_minimize_reduce_fcn():
@@ -292,32 +245,30 @@ def test_scalar_minimize_reduce_fcn():
 
     yo = 1.0 + 2.0*np.sin(4*x) * np.exp(-x / 5)
     y = yo + np.random.normal(size=len(yo), scale=0.250)
-    outliers = np.random.random_integers(int(len(x)/3.0), len(x)-1,
-                                         int(len(x)/12))
+    outliers = np.random.randint(int(len(x)/3.0), len(x), int(len(x)/12))
     y[outliers] += 5*np.random.random(len(outliers))
 
     # define objective function: returns the array to be minimized
     def objfunc(pars, x, data):
         decay = pars['decay']
-        offset= pars['offset']
+        offset = pars['offset']
         omega = pars['omega']
-        amp   = pars['amp']
+        amp = pars['amp']
         model = offset + amp * np.sin(x*omega) * np.exp(-x/decay)
         return model - data
 
     # create a set of Parameters
     params = Parameters()
     params.add('offset', 2.0)
-    params.add('omega',  3.3)
-    params.add('amp',    2.5)
-    params.add('decay',  1.0)
+    params.add('omega', 3.3)
+    params.add('amp', 2.5)
+    params.add('decay', 1.0)
 
-    method='L-BFGS-B'
+    method = 'L-BFGS-B'
     out1 = minimize(objfunc, params, args=(x, y), method=method)
     out2 = minimize(objfunc, params, args=(x, y), method=method,
                     reduce_fcn='neglogcauchy')
 
-    #print assert all
     assert_allclose(out1.params['omega'].value, 4.0, rtol=0.01)
     assert_allclose(out1.params['decay'].value, 7.6, rtol=0.01)
 
@@ -330,14 +281,14 @@ def test_multidimensional_fit_GH205():
     # function. Tests regression for GH205.
     pos = np.linspace(0, 99, 100)
     xv, yv = np.meshgrid(pos, pos)
-    f = lambda xv, yv, lambda1, lambda2: (np.sin(xv * lambda1)
-                                             + np.cos(yv * lambda2))
+    f = lambda xv, yv, lambda1, lambda2: (np.sin(xv * lambda1) +
+                                          np.cos(yv * lambda2))
 
     data = f(xv, yv, 0.3, 3)
-    assert_(data.ndim, 2)
+    assert data.ndim, 2
 
     def fcn2min(params, xv, yv, data):
-        """ model decaying sine wave, subtract data"""
+        """model decaying sine wave, subtract data"""
         model = f(xv, yv, params['lambda1'], params['lambda2'])
         return model - data
 
@@ -347,12 +298,11 @@ def test_multidimensional_fit_GH205():
     params.add('lambda2', value=3.2)
 
     mini = Minimizer(fcn2min, params, fcn_args=(xv, yv, data))
-    res = mini.minimize()
+    mini.minimize()
+
 
 def test_ufloat():
-    """
-    test of ufloat from uncertainties
-    """
+    """Test of ufloat from uncertainties."""
     x = ufloat(1, 0.1)
     assert_allclose(x.nominal_value, 1.0, rtol=1.e-7)
     assert_allclose(x.std_dev, 0.1, rtol=1.e-7)
@@ -366,8 +316,6 @@ def test_ufloat():
     assert_allclose(y.std_dev, 0.0, rtol=1.e-7)
 
 
-
-
 class CommonMinimizerTest(unittest.TestCase):
 
     def setUp(self):
@@ -392,10 +340,9 @@ class CommonMinimizerTest(unittest.TestCase):
         fit_params = Parameters()
         fit_params.add('amp', value=11.0, min=5, max=20)
         fit_params.add('period', value=5., min=1., max=7)
-        fit_params.add('shift', value=.10,  min=0.0, max=0.2)
+        fit_params.add('shift', value=.10, min=0.0, max=0.2)
         fit_params.add('decay', value=6.e-3, min=0, max=0.1)
         self.fit_params = fit_params
-
         self.mini = Minimizer(self.residual, fit_params, [self.x, self.data])
 
     def residual(self, pars, x, data=None):
@@ -417,7 +364,7 @@ class CommonMinimizerTest(unittest.TestCase):
         self.fit_params['decay'].min = -np.inf
         self.fit_params['decay'].vary = True
         self.minimizer = 'differential_evolution'
-        np.testing.assert_raises(ValueError, self.scalar_minimizer)
+        pytest.raises(ValueError, self.scalar_minimizer)
 
         # but only if a parameter is not fixed
         self.fit_params['decay'].vary = False
@@ -437,22 +384,11 @@ class CommonMinimizerTest(unittest.TestCase):
             self.scalar_minimizer(sig=sig)
 
     def scalar_minimizer(self, sig=0.15):
-        try:
-            from scipy.optimize import minimize as scipy_minimize
-        except ImportError:
-            raise SkipTest
-
-        print(self.minimizer)
         out = self.mini.scalar_minimize(method=self.minimizer)
 
         self.residual(out.params, self.x)
 
-        for name, par in out.params.items():
-            nout = "%s:%s" % (name, ' '*(20-len(name)))
-            print("%s: %s (%s) " % (nout, par.value, self.p_true[name].value))
-
-        for para, true_para in zip(out.params.values(),
-                                   self.p_true.values()):
+        for para, true_para in zip(out.params.values(), self.p_true.values()):
             check_wo_stderr(para, true_para.value, sig=sig)
 
     def test_nan_policy(self):
@@ -461,67 +397,74 @@ class CommonMinimizerTest(unittest.TestCase):
         self.data[0] = np.nan
 
         for method in SCALAR_METHODS:
-            assert_raises(ValueError,
-                          self.mini.scalar_minimize,
-                          SCALAR_METHODS[method])
+            if method == 'differential_evolution':
+                pytest.raises(RuntimeError, self.mini.scalar_minimize,
+                              SCALAR_METHODS[method])
+            else:
+                pytest.raises(ValueError, self.mini.scalar_minimize,
+                              SCALAR_METHODS[method])
 
-        assert_raises(ValueError, self.mini.minimize)
+        pytest.raises(ValueError, self.mini.minimize)
 
         # now check that the fit proceeds if nan_policy is 'omit'
         self.mini.nan_policy = 'omit'
         res = self.mini.minimize()
         assert_equal(res.ndata, np.size(self.data, 0) - 1)
 
-        for para, true_para in zip(res.params.values(),
-                                   self.p_true.values()):
+        for para, true_para in zip(res.params.values(), self.p_true.values()):
             check_wo_stderr(para, true_para.value, sig=0.15)
 
     def test_nan_policy_function(self):
         a = np.array([0, 1, 2, 3, np.nan])
-        assert_raises(ValueError, _nan_policy, a)
-        assert_(np.isnan(_nan_policy(a, nan_policy='propagate')[-1]))
+        pytest.raises(ValueError, _nan_policy, a)
+        assert np.isnan(_nan_policy(a, nan_policy='propagate')[-1])
         assert_equal(_nan_policy(a, nan_policy='omit'), [0, 1, 2, 3])
 
         a[-1] = np.inf
-        assert_raises(ValueError, _nan_policy, a)
-        assert_(np.isposinf(_nan_policy(a, nan_policy='propagate')[-1]))
+        pytest.raises(ValueError, _nan_policy, a)
+        assert np.isposinf(_nan_policy(a, nan_policy='propagate')[-1])
         assert_equal(_nan_policy(a, nan_policy='omit'), [0, 1, 2, 3])
         assert_equal(_nan_policy(a, handle_inf=False), a)
 
-    @decorators.slow
+    @dec.slow
     def test_emcee(self):
         # test emcee
         if not HAS_EMCEE:
             return True
 
         np.random.seed(123456)
-        out = self.mini.emcee(nwalkers=100, steps=200,
-                                      burn=50, thin=10)
+        out = self.mini.emcee(nwalkers=100, steps=200, burn=50, thin=10)
 
         check_paras(out.params, self.p_true, sig=3)
 
-    @decorators.slow
-    def test_emcee_PT(self):
-        # test emcee with parallel tempering
+    @dec.slow
+    def test_emcee_method_kwarg(self):
+        # test with emcee as method keyword argument
         if not HAS_EMCEE:
             return True
-
         np.random.seed(123456)
-        self.mini.userfcn = residual_for_multiprocessing
-        out = self.mini.emcee(ntemps=4, nwalkers=50, steps=200,
-                              burn=100, thin=10, workers=2)
+        out = self.mini.minimize(method='emcee',
+                                 nwalkers=50, steps=200,
+                                 burn=50, thin=10)
+        assert out.method == 'emcee'
+        assert out.nfev == 50*200
 
         check_paras(out.params, self.p_true, sig=3)
 
-    @decorators.slow
+        out_unweighted = self.mini.minimize(method='emcee',
+                                            nwalkers=50, steps=200,
+                                            burn=50, thin=10,
+                                            is_weighted=False)
+        assert out_unweighted.method == 'emcee'
+
+    @dec.slow
     def test_emcee_multiprocessing(self):
         # test multiprocessing runs
+        raise pytest.skip("Pytest fails with multiprocessing")
+        pytest.importorskip("dill")
         if not HAS_EMCEE:
             return True
-
-        np.random.seed(123456)
-        self.mini.userfcn = residual_for_multiprocessing
-        out = self.mini.emcee(steps=10, workers=4)
+        self.mini.emcee(steps=50, workers=4, nwalkers=20)
 
     def test_emcee_bounds_length(self):
         # the log-probability functions check if the parameters are
@@ -530,13 +473,13 @@ class CommonMinimizerTest(unittest.TestCase):
         # if nvarys != nparams
         if not HAS_EMCEE:
             return True
-        self.mini.params['amp'].vary=False
-        self.mini.params['period'].vary=False
-        self.mini.params['shift'].vary=False
+        self.mini.params['amp'].vary = False
+        self.mini.params['period'].vary = False
+        self.mini.params['shift'].vary = False
 
-        out = self.mini.emcee(steps=10)
+        self.mini.emcee(steps=10)
 
-    @decorators.slow
+    @dec.slow
     def test_emcee_partial_bounds(self):
         # mcmc with partial bounds
         if not HAS_EMCEE:
@@ -546,8 +489,7 @@ class CommonMinimizerTest(unittest.TestCase):
         # test mcmc output vs lm, some parameters not bounded
         self.fit_params['amp'].max = np.inf
         # self.fit_params['amp'].min = -np.inf
-        out = self.mini.emcee(nwalkers=100, steps=300,
-                                      burn=100, thin=10)
+        out = self.mini.emcee(nwalkers=100, steps=300, burn=100, thin=10)
 
         check_paras(out.params, self.p_true, sig=3)
 
@@ -558,37 +500,34 @@ class CommonMinimizerTest(unittest.TestCase):
 
         out = self.mini.emcee(nwalkers=100, steps=5)
         # can initialise with a chain
-        out2 = self.mini.emcee(nwalkers=100, steps=1, pos=out.chain)
-
+        self.mini.emcee(nwalkers=100, steps=1, pos=out.chain)
         # can initialise with a correct subset of a chain
-        out3 = self.mini.emcee(nwalkers=100,
-                               steps=1,
-                               pos=out.chain[..., -1, :])
+        self.mini.emcee(nwalkers=100, steps=1, pos=out.chain[-1, ...])
 
         # but you can't initialise if the shape is wrong.
-        assert_raises(ValueError,
+        pytest.raises(ValueError,
                       self.mini.emcee,
                       nwalkers=100,
                       steps=1,
-                      pos=out.chain[..., -1, :-1])
+                      pos=out.chain[-1, :-1, ...])
 
     def test_emcee_reuse_sampler(self):
         if not HAS_EMCEE:
             return True
 
-        self.mini.emcee(nwalkers=100, steps=5)
+        self.mini.emcee(nwalkers=20, steps=25)
 
         # if you've run the sampler the Minimizer object should have a _lastpos
         # attribute
-        assert_(hasattr(self.mini, '_lastpos'))
+        assert hasattr(self.mini, '_lastpos')
 
         # now try and re-use sampler
         out2 = self.mini.emcee(steps=10, reuse_sampler=True)
-        assert_(out2.chain.shape[1] == 15)
+        assert out2.chain.shape == (35, 20, 4)
 
         # you shouldn't be able to reuse the sampler if nvarys has changed.
         self.mini.params['amp'].vary = False
-        assert_raises(ValueError, self.mini.emcee, reuse_sampler=True)
+        pytest.raises(ValueError, self.mini.emcee, reuse_sampler=True)
 
     def test_emcee_lnpost(self):
         # check ln likelihood is calculated correctly. It should be
@@ -611,12 +550,10 @@ class CommonMinimizerTest(unittest.TestCase):
         # calculate the log-likelihood value
         bounds = np.array([(par.min, par.max)
                            for par in result.params.values()])
-        val2 = _lnpost(fvars,
-                       self.residual,
-                       result.params,
-                       result.var_names,
-                       bounds,
-                       userargs=(self.x, self.data))
+
+        val2 = self.mini._lnprob(fvars, self.residual, result.params,
+                                 result.var_names, bounds,
+                                 userargs=(self.x, self.data))
 
         assert_almost_equal(-0.5 * val, val2)
 
@@ -629,49 +566,25 @@ class CommonMinimizerTest(unittest.TestCase):
         except ImportError:
             return True
         out = self.mini.emcee(nwalkers=10, steps=20, burn=5, thin=2)
-        assert_(isinstance(out, MinimizerResult))
-        assert_(isinstance(out.flatchain, DataFrame))
+        assert isinstance(out, MinimizerResult)
+        assert isinstance(out.flatchain, DataFrame)
 
         # check that we can access the chains via parameter name
-        assert_(out.flatchain['amp'].shape[0] == 80)
-        assert_(out.errorbars is True)
-        assert_(np.isfinite(out.params['amp'].correl['period']))
+        # print( out.flatchain['amp'].shape[0],  200)
+        assert out.flatchain['amp'].shape[0] == 70
+        assert out.errorbars
+        assert np.isfinite(out.params['amp'].correl['period'])
 
         # the lnprob array should be the same as the chain size
-        assert_(np.size(out.chain)//out.nvarys == np.size(out.lnprob))
+        assert np.size(out.chain)//out.nvarys == np.size(out.lnprob)
 
         # test chain output shapes
-        assert_(out.lnprob.shape == (10, (20-5+1)/2) )
-        assert_(out.chain.shape == (10, (20-5+1)/2, out.nvarys) )
-        assert_(out.flatchain.shape == (10*(20-5+1)/2, out.nvarys))
+        print(out.lnprob.shape, out.chain.shape, out.flatchain.shape)
+        assert out.lnprob.shape == (7, 10)
+        assert out.chain.shape == (7, 10, 4)
+        assert out.flatchain.shape == (70, 4)
 
-    def test_emcee_PT_output(self):
-        # test mcmc output when using parallel tempering
-        if not HAS_EMCEE:
-            return True
-        try:
-            from pandas import DataFrame
-        except ImportError:
-            return True
-        out = self.mini.emcee(ntemps=6, nwalkers=10, steps=20, burn=5, thin=2)
-        assert_(isinstance(out, MinimizerResult))
-        assert_(isinstance(out.flatchain, DataFrame))
-
-        # check that we can access the chains via parameter name
-        assert_(out.flatchain['amp'].shape[0] == 80)
-        assert_(out.errorbars is True)
-        assert_(np.isfinite(out.params['amp'].correl['period']))
-
-        # the lnprob array should be the same as the chain size
-        assert_(np.size(out.chain)//out.nvarys == np.size(out.lnprob))
-
-        # test chain output shapes
-        assert_(out.lnprob.shape == (6, 10, (20-5+1)/2) )
-        assert_(out.chain.shape == (6, 10, (20-5+1)/2, out.nvarys) )
-        # Only the 0th temperature is returned
-        assert_(out.flatchain.shape == (10*(20-5+1)/2, out.nvarys))
-
-    @decorators.slow
+    @dec.slow
     def test_emcee_float(self):
         # test that it works if the residuals returns a float, not a vector
         if not HAS_EMCEE:
@@ -686,8 +599,7 @@ class CommonMinimizerTest(unittest.TestCase):
 
         self.mini.userfcn = resid
         np.random.seed(123456)
-        out = self.mini.emcee(nwalkers=100, steps=200,
-                                      burn=50, thin=10)
+        out = self.mini.emcee(nwalkers=100, steps=200, burn=50, thin=10)
         check_paras(out.params, self.p_true, sig=3)
 
         self.mini.userfcn = resid2
@@ -696,7 +608,7 @@ class CommonMinimizerTest(unittest.TestCase):
                               burn=50, thin=10, float_behavior='chi2')
         check_paras(out.params, self.p_true, sig=3)
 
-    @decorators.slow
+    @dec.slow
     def test_emcee_seed(self):
         # test emcee seeding can reproduce a sampling run
         if not HAS_EMCEE:
@@ -711,6 +623,14 @@ class CommonMinimizerTest(unittest.TestCase):
 
         assert_almost_equal(out.chain, out2.chain)
 
+    def test_emcee_ntemps(self):
+        # check for DeprecationWarning when using ntemps > 1
+        if not HAS_EMCEE:
+            return True
+
+        with pytest.raises(DeprecationWarning):
+            _ = self.mini.emcee(params=self.fit_params, ntemps=5)
+
 
 def residual_for_multiprocessing(pars, x, data=None):
     # a residual function defined in the top level is needed for
@@ -725,8 +645,4 @@ def residual_for_multiprocessing(pars, x, data=None):
     model = amp*np.sin(shift + x/per) * np.exp(-x*x*decay*decay)
     if data is None:
         return model
-    return (model - data)
-
-
-if __name__ == '__main__':
-    nose.main()
+    return model - data
